//	DevHndle.cpp

#include "DevHndle.hpp"

void showError( const char* _msg)
{
	LPVOID lpMsgBuf;
	FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
	    	NULL, GetLastError(), MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), 
	    	(LPTSTR) &lpMsgBuf, 0, NULL );
	// Display the string.
	//::MessageBox( NULL, lpMsgBuf, "GetLastError", MB_OK|MB_ICONINFORMATION );
    WMessageBox::Info( NULL, _msg, (char*)lpMsgBuf );
	LocalFree( lpMsgBuf );  // Free the buffer.
}
	

//////////////////////////////////////////////////////////////////////////////
//
// class	HandleW32 methods
//

HandleW32::HandleW32()	
	: hDev(INVALID_HANDLE_VALUE), hCount(0), name(),access(0),create(0),connected(false) 
{}

HandleW32::HandleW32( LPCTSTR _name, DWORD _access, DWORD _create )	
	: hDev(INVALID_HANDLE_VALUE), hCount(0),name(_name),access(_access),
	  create(_create),connected(false)
{
	Connect( _name, _access, _create );
}

HandleW32::HandleW32( const HandleW32& _dh ) 
//	: hDev(_dh.hDev), hCount(0), name(_dh.name),access(_dh.access),create(_dh.create)
{
//	if ( INVALID_HANDLE_VALUE != hDev )
//		hCount = _dh.hCount + 1;
}

HandleW32::~HandleW32() 
{
	if ( INVALID_HANDLE_VALUE != hDev )
	{ 
		// force closing of open handle
		hCount = 1;
		DisConnect(); 
	}
}	


void 
HandleW32::Connect( LPCTSTR _name, DWORD _access, DWORD _create )
{
	if ( 0 == hCount )
	{
		Set( _name, _access, _create );
		ConnectHandle();	
	}
	++hCount;	// increment reference count
}

void 
HandleW32::Connect()
{
	Connect( name, access, create );
}

void 
HandleW32::DisConnect()
{
	if ( 0 < hCount )
	{
		--hCount;	// decrement reference count
		if ( 0 == hCount )
		{
			// last reference has disconnected, so close the device handle
			CloseHandle(); 			// release device handle
		}
	}
}

void 
HandleW32::CloseHandle()
{
	::CloseHandle( hDev ); 			// release device
	hDev = INVALID_HANDLE_VALUE;	// not connected
	connected = false;
}

void 
HandleW32::ConnectHandle()
{
	hDev = CreateFile( name, access, NULL, NULL, create, NULL, NULL ); 
	if( INVALID_HANDLE_VALUE == hDev )
	{
		showError( name );
		throw HandleW32Exception();
	}
	connected = true;
}

void 
HandleW32::ReConnect()
{
	CloseHandle();
	ConnectHandle();
}

void		
HandleW32::Set( LPCTSTR _name, DWORD _access, DWORD _create )
{
	name =_name;
	access = _access;
	create = _create;
}

void		
HandleW32::Set( const HandleW32& _dh )
{
	name =_dh.name;
	access = _dh.access;
	create = _dh.create;
}

//////////////////////////////////////////////////////////////////////////////
//
// class	DevHandleW32 methods
//

DevHandleW32::DevHandleW32() 
	: hDev(0) 
{}

DevHandleW32::DevHandleW32( LPCTSTR _name, DWORD _access, DWORD _create ) 
	: hDev(0)
{
	Connect( _name, _access, _create );
}

DevHandleW32::DevHandleW32( const DevHandleW32& _dh ) 
	: hDev(0)
{
	Connect( _dh );
}

DevHandleW32::~DevHandleW32() 
{ 
	DisConnect(); 
}	


void 
DevHandleW32::Connect( LPCTSTR _name, DWORD _access, DWORD _create )
{
	if ( 0 == hDev )
	{
		hDev = new HandleW32( _name, _access, _create ); 
	}
	else
		throw DevHandleW32Exception();
}

void 
DevHandleW32::Connect( const DevHandleW32& _dh ) 
{
	if ( *this == _dh )
		return;
		
	if ( _dh.hDev && !hDev )
	{
		// we are not connected & _dh is connected
		hDev = _dh.hDev; 
		++hDev->hCount;
	}
	else
		// we are connected or_dh is not connected
		throw DevHandleW32Exception();
}

void 
DevHandleW32::DisConnect() 
{ 
	if ( hDev )
	{
		// hDev is connected to a device 
		--hDev->hCount;		// decrement it's reference count
		if ( 0 == hDev->hCount )
        {
			// hDev has no other connections, so delete it
			delete hDev;
        }
	    hDev = 0;       // no longer connected
	}
}	

void 
DevHandleW32::ReConnect()
{
	if ( hDev )
	{
		hDev->ReConnect();
	}
}

DevHandleW32::operator HANDLE () const { return Handle(); }

inline HANDLE 
DevHandleW32::Handle() const 
{ 
	if ( hDev ) 
		return hDev->Handle(); 
		else 
			return 0; 
}


