//	DevIf.cpp

#include "DevIf.hpp"		// for DevIf stuff
#include "wpch.hpp"

void
CommIf::SetPortSettings()
{
	HANDLE	hPort = DevHandle().Handle();	// handle to port
	
    //  Set time outs for read operations.
    COMMTIMEOUTS CommTimeout;
    if ( !GetCommTimeouts( hPort, &CommTimeout ) )
    {
        WMessageBox::Info( NULL, Name(),  "Failed to set comm Tiemouts." );
        return;
    }
    CommTimeout.ReadIntervalTimeout = 0;           
    CommTimeout.ReadTotalTimeoutMultiplier = 0;
    CommTimeout.ReadTotalTimeoutConstant = 2;
    CommTimeout.WriteTotalTimeoutMultiplier = 0;
    CommTimeout.WriteTotalTimeoutConstant = 0;
    if ( !SetCommTimeouts( hPort, &CommTimeout ) )
    {
        WMessageBox::Info( NULL, Name(),  "Failed to set comm Tiemouts." );
        return;
    }


    //  Fill Device Control Block for selected port.
    DCB   	dcb;            				// Device Control Block for port.
    if ( !GetCommState( hPort, &dcb ) )
    {
        WMessageBox::Info( NULL, Name(),  "Failed to get comm State." );
        return;
    }

    //  Modify Device Control Block to our settings.
    // 1st the baud rate
    switch( settings.BaudRate() )
    {
    case 300:
        dcb.BaudRate = CBR_300;
        break;
    case 1200:
        dcb.BaudRate = CBR_1200;
        break;
    case 2400:
        dcb.BaudRate = CBR_2400;
        break;
    case 4800:
        dcb.BaudRate = CBR_4800;
        break;
    case 9600:
        dcb.BaudRate = CBR_9600;
        break;
    case 14400:
        dcb.BaudRate = CBR_14400;
        break;
    case 19200:
        dcb.BaudRate = CBR_19200;
        break;
    case 38400:
        dcb.BaudRate = CBR_38400;
        break;
    case 56000:
        dcb.BaudRate = CBR_56000;
        break;
    case 57600:
        dcb.BaudRate = CBR_57600;
        break;
    case 115200:
        dcb.BaudRate = CBR_115200;
        break;
    case 128000:
        dcb.BaudRate = CBR_128000;
        break;
    case 256000:
        dcb.BaudRate = CBR_256000;
        break;
    default:
        dcb.BaudRate = CBR_9600;
    }

	// next the no. of data bits
    dcb.ByteSize = (BYTE) settings.DataBits();
    
    // then the parity
    switch( settings.Parity() )
    {
    case 0:
    	dcb.Parity   = NOPARITY;
    	break;
    case 1:
    	dcb.Parity   = ODDPARITY;
    	break;
    case 2:
    	dcb.Parity   = EVENPARITY;
    	break;
    case 3:
    	dcb.Parity   = MARKPARITY;
    	break;
	default:
    	dcb.Parity   = NOPARITY;
    	break;
    }
    
    // and finally, the no. of stop bits
    switch( settings.StopBits() )
    {
    case 1:
    	dcb.StopBits   = ONESTOPBIT;
    	break;
    case 2:
    	dcb.StopBits   = TWOSTOPBITS;
    	break;
    case 3:
    	dcb.StopBits   = ONE5STOPBITS;
    	break;
	default:
    	dcb.StopBits   = ONESTOPBIT;
    	break;
    }

    //  Now, set the device to our settings.
    if( !SetCommState( hPort, &dcb ))
    {
      	WMessageBox::Info( NULL, Name(), "Failed to set Comm State." );
        return;
    }
    
}

void
CommIf::SetPortSettingsX()
{
    DCB   	dcb;            				// Device Control Block for port.
	HANDLE	hPort = DevHandle().Handle();	// handle to port
	
    //  Set time outs for read operations.
    COMMTIMEOUTS CommTimeout;
    GetCommTimeouts( hPort, &CommTimeout );
    CommTimeout.ReadIntervalTimeout = MAXDWORD;           // max mS between chars
    CommTimeout.ReadTotalTimeoutMultiplier = 0;
    CommTimeout.ReadTotalTimeoutConstant = 0;
    CommTimeout.WriteTotalTimeoutMultiplier = 0;
    CommTimeout.WriteTotalTimeoutConstant = 0;
    SetCommTimeouts( hPort, &CommTimeout );

    //  Fill Device Control Block for selected port.
    if ( !GetCommState( hPort, &dcb ) )
    {
        WMessageBox::Info( NULL, "port",  "Failed to get comm State." );
        return;
    }

    //  Modify Device Control Block to our settings.
    // 1st the baud rate
    switch( settings.BaudRate() )
    {
    case 300:
        dcb.BaudRate = CBR_300;
        break;
    case 1200:
        dcb.BaudRate = CBR_1200;
        break;
    case 2400:
        dcb.BaudRate = CBR_2400;
        break;
    case 4800:
        dcb.BaudRate = CBR_4800;
        break;
    case 9600:
        dcb.BaudRate = CBR_9600;
        break;
    case 14400:
        dcb.BaudRate = CBR_14400;
        break;
    case 19200:
        dcb.BaudRate = CBR_19200;
        break;
    case 38400:
        dcb.BaudRate = CBR_38400;
        break;
    case 56000:
        dcb.BaudRate = CBR_56000;
        break;
    case 115200:
        dcb.BaudRate = CBR_115200;
        break;
    default:
        dcb.BaudRate = CBR_9600;
    }

	// next the no. of data bits
    dcb.ByteSize = (BYTE) settings.DataBits();
    
    // then the parity
    switch( settings.Parity() )
    {
    case 0:
    	dcb.Parity   = NOPARITY;
    	break;
    case 1:
    	dcb.Parity   = ODDPARITY;
    	break;
    case 2:
    	dcb.Parity   = EVENPARITY;
    	break;
    case 3:
    	dcb.Parity   = MARKPARITY;
    	break;
	default:
    	dcb.Parity   = NOPARITY;
    	break;
    }
    
    // and finally, the no. of stop bits
    switch( settings.StopBits() )
    {
    case 1:
    	dcb.StopBits   = ONESTOPBIT;
    	break;
    case 2:
    	dcb.StopBits   = TWOSTOPBITS;
    	break;
    case 3:
    	dcb.StopBits   = ONE5STOPBITS;
    	break;
	default:
    	dcb.StopBits   = ONESTOPBIT;
    	break;
    }

    //  Now, set the device to our settings.
    if( !SetCommState( hPort, &dcb ))
    {
      	WMessageBox::Info( NULL, "port", "Failed to set Comm State." );
        return;
    }
    
}

