//	DevIf.hpp

#ifndef __DEVIF_HPP__
#define __DEVIF_HPP__

#include "DevHndle.hpp"
#include "PortSet.hpp"

//////////////////////////////////////////////////////////////////////////////
//
// class	DevIf
//
class	DevIf
{
public:
	virtual	~DevIf() {}
	
	virtual	void Connect( LPCTSTR _name, DWORD _access, DWORD _create ) =0;
	virtual	void Connect( const DevIf& _di ) =0;
	virtual void ReConnect() =0;
	virtual void DisConnect() =0;
	
	virtual const DevHandleW32&		DevHandle() const =0;

};

//////////////////////////////////////////////////////////////////////////////
//
// class	RxDevIf
//
class	RxDevIf : public DevIf
{
public:
	virtual	void Connect( LPCTSTR _name, DWORD _access, DWORD _create ) =0;
	virtual	void Connect( const DevIf& _di ) =0;
	virtual void ReConnect() =0;
	virtual void DisConnect() =0;
	virtual const DevHandleW32&		DevHandle() const =0;

	virtual DWORD	ReadData( char* _dst, DWORD _amount )
	{
		DWORD	br;
		bool ok = ReadFile( DevHandle(), _dst, _amount, &br, NULL );
		if ( !ok )
		{
			showError("RxDevIf::ReadData()");
		}
		return br;	
	}
	
};



//////////////////////////////////////////////////////////////////////////////
//
// class	TxDevIf
//
class	TxDevIf  : public DevIf
{
public:
	virtual	void Connect( LPCTSTR _name, DWORD _access, DWORD _create ) =0;
	virtual	void Connect( const DevIf& _di ) =0;
	virtual const DevHandleW32&		DevHandle() const =0;
	virtual void ReConnect() =0;
	virtual void DisConnect() =0;

	virtual DWORD	WriteData( char* _src, DWORD _amount )
	{
		DWORD	br;
		bool ok = WriteFile( DevHandle(), _src, _amount, &br, NULL );
		if ( !ok )
		{
			showError("TxDevIf::WriteData()");
		}
		return br;	
	}
	
	
};



//////////////////////////////////////////////////////////////////////////////
//
// class	FileIf
//
class	FileIf
{
public:
	FileIf() 
	{
	}
	
	FileIf( LPCTSTR _name, DWORD _access,	DWORD _create )
		: devHandle( _name, _access, _create )
	{
	}
	
	explicit FileIf( const DevHandleW32& _dh )
		: devHandle( _dh ) 
	{
	}

	virtual	~FileIf() {}
	
	void Connect( LPCTSTR _name, DWORD _access, DWORD _create )
	{
        WCurrentThread  th;
        
        if ( devHandle.Connected() )
        {
            DisConnect();		// 1st disconnect from current handle
            th.Wait( 250 );		// wait for op sys to fully release handle
        }
        // now connect to requested handle
		devHandle.Connect( _name, _access, _create );
	}

	void Connect( const DevIf& _di )
	{
        WCurrentThread  th;
        
        if ( devHandle.Connected() )
        {
            DisConnect();		// 1st disconnect from current handle
            th.Wait( 250 );		// wait for op sys to fully release handle
        }
		devHandle.Connect( _di.DevHandle() );
	}
		
	void DisConnect()
	{
		devHandle.DisConnect();
	}

	void ReConnect()
	{
		devHandle.ReConnect();
	}
		
	const DevHandleW32&		DevHandle() const 
	{ 
		return devHandle; 
	}

	//operator 	HANDLE () const { return devHandle.Handle(); }
	HANDLE 		Handle() const { return devHandle.Handle(); }
	LPCTSTR		Name() const { return devHandle.Name(); }
	DWORD		AccessMode() const { return	devHandle.AccessMode(); }
	DWORD		CreateMode() const { return	devHandle.CreateMode(); }
	bool		Connected() const { return	devHandle.Connected(); }
	
	
private:
	DevHandleW32	devHandle;
};


//////////////////////////////////////////////////////////////////////////////
//
// class	RxFileDevIf
//
class	RxFileDevIf : public RxDevIf, private FileIf
{
public:
	RxFileDevIf() : FileIf() {}
	RxFileDevIf( LPCTSTR _name, DWORD _access,	DWORD _create )
		: FileIf( _name, _access, _create ) 
	{
	}	
	explicit RxFileDevIf( const DevHandleW32& _dh )
			: FileIf( _dh ) 
	{
	}	

	virtual	~RxFileDevIf() {}

	virtual	void Connect( LPCTSTR _name, DWORD _access, DWORD _create )
	{
		FileIf::Connect( _name, _access, _create );
	}
	
	virtual	void Connect( const DevIf& _di )
	{
		FileIf::Connect( _di );
	}
	
	virtual void ReConnect()
	{
		FileIf::ReConnect();
	}
	
	virtual void DisConnect()
	{
		FileIf::DisConnect();
	}
	
	const DevHandleW32&		DevHandle() const { return FileIf::DevHandle(); }

	LPCTSTR		Name() const { return FileIf::Name(); }
	DWORD		AccessMode() const { return	FileIf::AccessMode(); }
	DWORD		CreateMode() const { return	FileIf::CreateMode(); }
	bool		Connected() const { return FileIf::Connected(); }

};



//////////////////////////////////////////////////////////////////////////////
//
// class	TxFileDevIf
//
class	TxFileDevIf : public TxDevIf, private FileIf
{
public:
	TxFileDevIf() : FileIf() {}
	TxFileDevIf( LPCTSTR _name, DWORD _access,	DWORD _create )
		: FileIf( _name, _access, _create ) 
	{
	}	
	explicit TxFileDevIf( const DevHandleW32& _dh )
			: FileIf( _dh ) 
	{
	}	

	virtual	~TxFileDevIf() {}

	virtual	void Connect( LPCTSTR _name, DWORD _access, DWORD _create )
	{
		FileIf::Connect( _name, _access, _create );
	}
	
	virtual	void Connect( const DevIf& _di )
	{
		FileIf::Connect( _di );
	}
	
	virtual void ReConnect()
	{
		FileIf::ReConnect();
	}
	
	virtual void DisConnect()
	{
		FileIf::DisConnect();
	}

	const DevHandleW32&		DevHandle() const { return FileIf::DevHandle(); }

	LPCTSTR		Name() const { return FileIf::Name(); }
	DWORD		AccessMode() const { return	FileIf::AccessMode(); }
	DWORD		CreateMode() const { return	FileIf::CreateMode(); }
	bool					Connected() const { return FileIf::Connected(); }

};


//////////////////////////////////////////////////////////////////////////////
//
// class	CommIf
//
class	CommIf: public FileIf
{
public:
	CommIf() : FileIf(), settings() {}
	CommIf( LPCTSTR _name, DWORD _access,  uint _br, uint _db, uint _p, uint _sb )
		: FileIf(), settings(_br,_db,_p,_sb)	
	{
		Connect( _name, _access,  settings );
	}
	
	explicit CommIf( DevHandleW32 const & _dh ) : FileIf(_dh), settings() {}
	CommIf( LPCTSTR _name, DWORD _access,  char* _str )
		: FileIf(), settings(_str)	
	{
		Connect( _name, _access,  settings );
	}
	
	CommIf( LPCTSTR _name, DWORD _access,  PortSettings const & _ps )
		: FileIf(), settings(_ps)	
	{
		Connect( _name, _access,  settings );
	}
	
	virtual	~CommIf() {}
	
	virtual	void Connect( LPCTSTR _name, DWORD _access, DWORD )
	{
		FileIf::Connect( _name, _access, OPEN_EXISTING );
		//settings.ReSet();
		SetPortSettings();
	}
	
	void Connect( const DevIf& _di )
	{
		FileIf::Connect( _di );
		//settings.ReSet();
		SetPortSettings();
	}
	
	void Connect( LPCTSTR _name, DWORD _access,  uint _br, uint _db, uint _p, uint _sb )
	{
		FileIf::Connect( _name, _access, OPEN_EXISTING );
		SetPortSettings( _br, _db, _p, _sb );
	}
	
	void Connect( LPCTSTR _name, DWORD _access,  char const * _str )
	{
		FileIf::Connect( _name, _access, OPEN_EXISTING );
		SetPortSettings( _str );
	}
	
	void Connect( LPCTSTR _name, DWORD _access,  PortSettings const & _ps )
	{
		FileIf::Connect( _name, _access, OPEN_EXISTING );
		SetPortSettings( _ps );
	}
	
	void SetPortSettings( uint _br, uint _db, uint _p, uint _sb )
	{
		settings.Set( _br, _db, _p, _sb );
		SetPortSettings();
	}
	
	void SetPortSettings( char const * _str )
	{
		settings.Set( _str );
		SetPortSettings();
	}
	
	void SetPortSettings( PortSettings const & _ps )
	{
		settings = _ps;
		SetPortSettings();
	}

	void SetPortSettings();
	void SetPortSettingsX();
		
private:
	PortSettings			settings;
};

typedef	PortSettings	PS;

//////////////////////////////////////////////////////////////////////////////
//
// class	RxCommDevIf
//
class	RxCommDevIf : public RxDevIf, private CommIf
{
public:
	RxCommDevIf() : CommIf() 
	{
	}

	explicit RxCommDevIf( const DevHandleW32& _dh )
			: CommIf( _dh ) 
	{
	}	

	RxCommDevIf( LPCTSTR _name, DWORD _access )
		: CommIf( _name, _access,  PS::DefBaud, PS::DefDB, PS::DefPar, PS::DefSB )	
	{
	}
	
	RxCommDevIf( LPCTSTR _name, DWORD _access, uint _br, uint _db, uint _p, uint _sb )
		: CommIf( _name, _access,  _br, _db, _p, _sb )	
	{
	}
	
	RxCommDevIf( LPCTSTR _name, DWORD _access,  char* _str )
		: CommIf( _name, _access,  _str )	
	{
	}
	
	RxCommDevIf( LPCTSTR _name, DWORD _access,  PortSettings const & _ps )
		: CommIf( _name, _access,  _ps )	
	{
	}

	virtual	~RxCommDevIf() {}

	virtual	void Connect( LPCTSTR _name, DWORD _access, DWORD _create )
	{
		CommIf::Connect( _name, _access, _create );
	}
	
	virtual	void Connect( const DevIf& _di )
	{
		CommIf::Connect( _di );
	}
	
	void Connect( LPCTSTR _name, DWORD _access,  uint _br, uint _db, uint _p, uint _sb )
	{
		CommIf::Connect(  _name, _access,  _br, _db, _p, _sb  );
	}
	
	void Connect( LPCTSTR _name, DWORD _access,  char const * _str )
	{
		CommIf::Connect(  _name, _access,  _str  );
	}
	
	void Connect( LPCTSTR _name, DWORD _access,  PortSettings const & _ps )
	{
		CommIf::Connect(  _name, _access,  _ps  );
	}
	
	virtual void ReConnect()
	{
		CommIf::ReConnect();
	}
	
	virtual void DisConnect()
	{
		CommIf::DisConnect();
	}

	void SetPortSettings( uint _br, uint _db, uint _p, uint _sb )
	{
		CommIf::SetPortSettings( _br, _db, _p, _sb );
	}
	
	void SetPortSettings( char const * _str )
	{
		CommIf::SetPortSettings( _str );
	}
	
	void SetPortSettings( PortSettings const & _ps )
	{
		CommIf::SetPortSettings( _ps );
	}

	LPCTSTR		Name() const { return CommIf::Name(); }
	DWORD		AccessMode() const { return	CommIf::AccessMode(); }
	DWORD		CreateMode() const { return	CommIf::CreateMode(); }
	bool					Connected() const { return CommIf::Connected(); }

	const DevHandleW32&		DevHandle() const { return CommIf::DevHandle(); }

};



//////////////////////////////////////////////////////////////////////////////
//
// class	TxCommDevIf
//
class	TxCommDevIf : public TxDevIf, private CommIf
{
public:
	TxCommDevIf() : CommIf() 
	{
	}

	explicit TxCommDevIf( const DevHandleW32& _dh )
			: CommIf( _dh ) 
	{
	}	

	TxCommDevIf( LPCTSTR _name, DWORD _access )
		: CommIf( _name, _access,  PS::DefBaud, PS::DefDB, PS::DefPar, PS::DefSB )	
	{
	}
	
	TxCommDevIf( LPCTSTR _name, DWORD _access,  uint _br, uint _db, uint _p, uint _sb )
		: CommIf( _name, _access,  _br, _db, _p, _sb )	
	{
	}
	
	TxCommDevIf( LPCTSTR _name, DWORD _access,  char* _str )
		: CommIf( _name, _access,  _str )	
	{
	}
	
	TxCommDevIf( LPCTSTR _name, DWORD _access,  PortSettings const & _ps )
		: CommIf( _name, _access,  _ps )	
	{
	}

	virtual	~TxCommDevIf() {}

	virtual	void Connect( LPCTSTR _name, DWORD _access, DWORD _create )
	{
		CommIf::Connect( _name, _access, _create );
	}
	
	virtual	void Connect( const DevIf& _di )
	{
		CommIf::Connect( _di );
	}
	
	void Connect( LPCTSTR _name, DWORD _access,  uint _br, uint _db, uint _p, uint _sb )
	{
		CommIf::Connect(  _name, _access,  _br, _db, _p, _sb  );
	}
	
	void Connect( LPCTSTR _name, DWORD _access,  char const * _str )
	{
		CommIf::Connect(  _name, _access,  _str  );
	}
	
	void Connect( LPCTSTR _name, DWORD _access,  PortSettings const & _ps )
	{
		CommIf::Connect(  _name, _access,  _ps  );
	}
	
	virtual void ReConnect()
	{
		CommIf::ReConnect();
	}
	
	virtual void DisConnect()
	{
		CommIf::DisConnect();
	}

	void SetPortSettings( uint _br, uint _db, uint _p, uint _sb )
	{
		CommIf::SetPortSettings( _br, _db, _p, _sb );
	}
	
	void SetPortSettings( char const * _str )
	{
		CommIf::SetPortSettings( _str );
	}
	
	void SetPortSettings( PortSettings const & _ps )
	{
		CommIf::SetPortSettings( _ps );
	}

	LPCTSTR		Name() const { return CommIf::Name(); }
	DWORD		AccessMode() const { return	CommIf::AccessMode(); }
	DWORD		CreateMode() const { return	CommIf::CreateMode(); }
	bool		Connected() const { return CommIf::Connected(); }

	const DevHandleW32&		DevHandle() const { return FileIf::DevHandle(); }

};



#include "DevIf.cpp"		// until create library

#endif	// __DEVIF_HPP__
