//	IoDev.cpp


#include "IoDev.hpp"

//////////////////////////////////////////////////////////////////////////////
//
// class	IoDev methods
//

IoDev::IoDev( uint _bsze ) : bufSize(_bsze), buf(0)
{
	SetBuf( bufSize );
}

IoDev::~IoDev() 
{ 
	delete buf; 
} 

void	
IoDev::SetBuf( uint _bufSize )
{
	bufSize = _bufSize;
	if ( buf )
		delete buf;
	buf = new char[bufSize];
	if ( !buf )
		bufSize = 0;
}

char*	
IoDev::Buf() const 
{ 
	return buf; 
}

uint	
IoDev::BufSize() const  
{ 
	return bufSize; 
}

//////////////////////////////////////////////////////////////////////////////
//
// class	FileRxDev methods
//

FileRxDev::FileRxDev( uint _qsze )
	: IoDev(_qsze/4), rxDevIf(), rxQue(_qsze)
{
}

FileRxDev::FileRxDev(  const DevIf& _di , uint _qsze )
	: IoDev(_qsze/4), rxDevIf(_di.DevHandle()), rxQue(_qsze)
    {
    }

FileRxDev::FileRxDev( LPCTSTR _name, DWORD _access, DWORD _create, uint _qsze )
	: IoDev(_qsze/4), rxDevIf(_name,_access,_create), rxQue(_qsze)
{
}

FileRxDev::FileRxDev( const FileRxDev& _dev )
	: IoDev(_dev.BufSize()), rxDevIf(_dev.rxDevIf.DevHandle()), rxQue(_dev.rxQue.Capacity())
{
}

FileRxDev::~FileRxDev() {}

void 
FileRxDev::Connect( LPCTSTR _name, DWORD _access, DWORD _create )
{
	rxDevIf.Connect( _name, _access, _create );
}
void 
FileRxDev::Connect( const IoDev& _iod )
{
	rxDevIf.Connect( _iod.DevInterface() );
}

void 	
FileRxDev::Connect( const DevIf& _di )
{
	rxDevIf.Connect( _di );
}

char const & 
FileRxDev::Remove() 
{ 
	ReadData();
	return rxQue.Remove(); 
}

uint   		
FileRxDev::Remove( char* _dest, uint _qty ) 
{ 
	ReadData();
	return rxQue.Remove( _dest, _qty ); 
}

DevIf& 
FileRxDev::DevInterface() const { return (DevIf&)rxDevIf; }

inline
char const & 
FileRxDev::RemoveX() 
{ 
	return rxQue.Remove(); 
}

inline
uint   		
FileRxDev::RemoveX( char* _dest, uint _qty ) 
{ 
	return rxQue.Remove( _dest, _qty ); 
}

void	
FileRxDev::ReadData()
{
	long	num = rxDevIf.ReadData( Buf(), BufSize() );
	rxQue.Add( Buf(), num );
}


//////////////////////////////////////////////////////////////////////////////
//
// class	FileTxDev methods
//

FileTxDev::FileTxDev( uint _qsze )
	: IoDev(_qsze/4), txDevIf(), txQue(_qsze)
{
}

FileTxDev::FileTxDev(  const DevIf& _di , uint _qsze )
	: IoDev(_qsze/4), txDevIf(_di.DevHandle()), txQue(_qsze)
{
}

FileTxDev::FileTxDev( LPCTSTR _name, DWORD _access, DWORD _create, uint _qsze )
	: IoDev(_qsze/4), txDevIf(_name,_access,_create), txQue(_qsze)
{
}

FileTxDev::FileTxDev( const FileTxDev& _dev )
	: IoDev(_dev.BufSize()), txDevIf(_dev.txDevIf.DevHandle()), txQue(_dev.txQue.Capacity())
{
}

FileTxDev::~FileTxDev() 
{}

void 
FileTxDev::Connect( LPCTSTR _name, DWORD _access, DWORD _create )
{
	txDevIf.Connect( _name, _access, _create );
}

void 
FileTxDev::Connect( const IoDev& _iod )
{
	txDevIf.Connect( _iod.DevInterface() );
}

void 	
FileTxDev::Connect( const DevIf& _di )
{
	txDevIf.Connect( _di );
}

const char& 
FileTxDev::Add( const char& _item ) 
{ 
	const char& rem = txQue.Add( _item ); 
	WriteData();
	return rem;
}

uint	    
FileTxDev::Add( const char* _src, uint _qty ) 
{ 
	uint num = txQue.Add( (char*)_src, _qty ); 
	WriteData();
	return num;
}

inline
const char& 
FileTxDev::AddX( const char& _item ) 
{ 
	return txQue.Add( _item ); 
}

inline
uint	    
FileTxDev::AddX( const char* _src, uint _qty ) 
{ 
	return txQue.Add( (char*)_src, _qty ); 
}

inline
DevIf& 
FileTxDev::DevInterface() const 
{ 
	return (DevIf&)txDevIf; 
}

void	
FileTxDev::WriteData()
{
	long	num = txQue.Remove( Buf(), BufSize() );
	txDevIf.WriteData( Buf(), num );
	
}


//////////////////////////////////////////////////////////////////////////////
//
// class	CommRxDev methods
//

CommRxDev::CommRxDev( uint _qsze )
	: IoDev(16), rxDevIf(), rxQue(_qsze)
{
}

CommRxDev::CommRxDev(  const DevIf& _di , uint _qsze )
	: IoDev(16), rxDevIf(_di.DevHandle()), rxQue(_qsze)
    {
    }

CommRxDev::CommRxDev( LPCTSTR _name, DWORD _access, DWORD, uint _qsze )
	: IoDev(16), rxDevIf(_name,_access), rxQue(_qsze)
{
}

CommRxDev::CommRxDev( const CommRxDev& _dev )
	: IoDev(_dev.BufSize()), rxDevIf(_dev.rxDevIf.DevHandle()), rxQue(_dev.rxQue.Capacity())
{
}

CommRxDev::CommRxDev( LPCTSTR _name, DWORD _access,  uint _br, uint _db, uint _p, uint _sb, uint _qsze )
	: IoDev(16), rxDevIf(_name,_access,_br,_db,_p,_sb), rxQue(_qsze)
{
}

CommRxDev::CommRxDev( LPCTSTR _name, DWORD _access,  char* _str, uint _qsze )
	: IoDev(16), rxDevIf(_name,_access,_str), rxQue(_qsze)
{
}

CommRxDev::CommRxDev( LPCTSTR _name, DWORD _access,  PortSettings const & _ps, uint _qsze )
	: IoDev(16), rxDevIf(_name,_access,_ps), rxQue(_qsze)
{
}



CommRxDev::~CommRxDev() {}

void 	
CommRxDev::Connect( const DevIf& _di )
{
	rxDevIf.Connect( _di );
}

void 
CommRxDev::Connect( const IoDev& _iod )
{
	rxDevIf.Connect( _iod.DevInterface() );
}

void 
CommRxDev::Connect( LPCTSTR _name, DWORD _access, DWORD _create )
{
	rxDevIf.Connect( _name, _access, _create );
}

void 
CommRxDev::Connect( LPCTSTR _name, DWORD _access, uint _br, uint _db, uint _p, uint _sb )
{
	rxDevIf.Connect( _name, _access, _br, _db, _p, _sb );
}

void 
CommRxDev::Connect( LPCTSTR _name, DWORD _access,  char const * _str )
{
	rxDevIf.Connect( _name, _access, _str );
}

void 
CommRxDev::Connect( LPCTSTR _name, DWORD _access,  PortSettings const & _ps )
{
	rxDevIf.Connect( _name, _access, _ps );
}


const 
char& 
CommRxDev::Remove() 
{ 
	ReadData();
	return rxQue.Remove(); 
}

uint   		
CommRxDev::Remove( char* _dest, uint _qty ) 
{ 
	ReadData();
	return rxQue.Remove( _dest, _qty ); 
}

DevIf& 
CommRxDev::DevInterface() const { return (DevIf&)rxDevIf; }

inline
char const & 
CommRxDev::RemoveX() 
{ 
	return rxQue.Remove(); 
}

inline
uint   		
CommRxDev::RemoveX( char* _dest, uint _qty ) 
{ 
	return rxQue.Remove( _dest, _qty ); 
}

void	
CommRxDev::ReadData()
{
	long	num = rxDevIf.ReadData( Buf(), BufSize() );
	rxQue.Add( Buf(), num );
}

//////////////////////////////////////////////////////////////////////////////
//
// class	CommTxDev methods
//

CommTxDev::CommTxDev( uint _qsze )
	: IoDev(16), txDevIf(), txQue(_qsze)
{
}

CommTxDev::CommTxDev(  const DevIf& _di , uint _qsze )
	: IoDev(16), txDevIf(_di.DevHandle()), txQue(_qsze)
    {
    }

CommTxDev::CommTxDev( LPCTSTR _name, DWORD _access, DWORD, uint _qsze )
	: IoDev(16), txDevIf(_name,_access), txQue(_qsze)
{
}

CommTxDev::CommTxDev( const CommTxDev& _dev )
	: IoDev(_dev.BufSize()), txDevIf(_dev.txDevIf.DevHandle()), txQue(_dev.txQue.Capacity())
{
}

CommTxDev::CommTxDev( LPCTSTR _name, DWORD _access,  uint _br, uint _db, uint _p, uint _sb, uint _qsze )
	: IoDev(16), txDevIf(_name,_access,_br,_db,_p,_sb), txQue(_qsze)
{
}

CommTxDev::CommTxDev( LPCTSTR _name, DWORD _access,  char* _str, uint _qsze )
	: IoDev(16), txDevIf(_name,_access,_str), txQue(_qsze)
{
}

CommTxDev::CommTxDev( LPCTSTR _name, DWORD _access,  PortSettings const & _ps, uint _qsze )
	: IoDev(16), txDevIf(_name,_access,_ps), txQue(_qsze)
{
}



CommTxDev::~CommTxDev() {}

void 	
CommTxDev::Connect( const DevIf& _di )
{
	txDevIf.Connect( _di );
}

void 
CommTxDev::Connect( const IoDev& _iod )
{
	txDevIf.Connect( _iod.DevInterface() );
}

void 
CommTxDev::Connect( LPCTSTR _name, DWORD _access, DWORD _create )
{
	txDevIf.Connect( _name, _access, _create );
}

void 
CommTxDev::Connect( LPCTSTR _name, DWORD _access, uint _br, uint _db, uint _p, uint _sb )
{
	txDevIf.Connect( _name, _access, _br, _db, _p, _sb );
}

void 
CommTxDev::Connect( LPCTSTR _name, DWORD _access,  char const * _str )
{
	txDevIf.Connect( _name, _access, _str );
}

void 
CommTxDev::Connect( LPCTSTR _name, DWORD _access,  PortSettings const & _ps )
{
	txDevIf.Connect( _name, _access, _ps );
}


const char& 
CommTxDev::Add( const char& _item ) 
{ 
	const char& rem = txQue.Add( _item ); 
	WriteData();
	return rem;
}

uint	    
CommTxDev::Add( const char* _src, uint _qty ) 
{ 
	uint num = txQue.Add( (char*)_src, _qty ); 
	WriteData();
	return num;
}

inline
DevIf& 
CommTxDev::DevInterface() const 
{ 
	return (DevIf&)txDevIf; 
}

inline
const char& 
CommTxDev::AddX( const char& _item ) 
{ 
	return txQue.Add( _item ); 
}

inline
uint	    
CommTxDev::AddX( const char* _src, uint _qty ) 
{ 
	return txQue.Add( (char*)_src, _qty ); 
}

void	
CommTxDev::WriteData()
{
	long	num = txQue.Remove( Buf(), BufSize() );

	if ( 0 < num )
		txDevIf.WriteData( Buf(), num );
	
}


//////////////////////////////////////////////////////////////////////////////
//
// class	CommRxDevTh methods
//

CommRxDevTh::CommRxDevTh( uint _qsze )
	: CommRxDev(_qsze)
{
	Init( Connected() );
}

CommRxDevTh::CommRxDevTh(  const DevIf& _di, uint _qsze )
	: CommRxDev(_di,_qsze)
{
	Init( Connected() );
}

CommRxDevTh::CommRxDevTh( LPCTSTR _name, DWORD _access, DWORD, uint _qsze )
	: CommRxDev(_name,_access, OPEN_EXISTING,_qsze)
{
	Init( Connected() );
}

CommRxDevTh::CommRxDevTh( const CommRxDevTh& _dev )
	: CommRxDev(_dev)
{
	Init( Connected() );
}

CommRxDevTh::CommRxDevTh( LPCTSTR _name, DWORD _access,  uint _br, uint _db, uint _p, uint _sb, uint _qsze )
	: CommRxDev(_name,_access,_br,_db,_p,_sb,_qsze)
{
	Init( Connected() );
}

CommRxDevTh::CommRxDevTh( LPCTSTR _name, DWORD _access, char* _str, uint _qsze )
	: CommRxDev(_name,_access,_str,_qsze)
{
	Init( Connected() );
}

CommRxDevTh::CommRxDevTh( LPCTSTR _name, DWORD _access, PortSettings const & _ps, uint _qsze )
	: CommRxDev(_name,_access,_ps,_qsze)
{
	Init( Connected() );
}

void
CommRxDevTh::Init( bool _connected ) 
{
	// create the semaphores 
	//WString	ws = Name();
    grabQue.Create( ""/*ws + "RxgrabQue"*/ );				
    portTrafficLight.Create( ""/*ws + "RxTrafficLight"*/ );	
    	
	running = true;		// start off running
	
	// set traffic light according to connected status
	SetGreenLight( Connected() );
    //WCurrentThread  th; th.Wait( 500 );		// wait for op sys 
	
	// start the port access thread
	readThread.Create( this, (WThreadRoutine)ReadDataTh, 0 );
	readThread.SetPriority( WThreadPriorityAboveNormal );
}

CommRxDevTh::~CommRxDevTh() 
{
	running = false;			// tell thread to shutdown
    portTrafficLight.Set();    	// stop thread waiting on semaphore
	readThread.Wait();			// wait for thread to shutdown
}

void 	
CommRxDevTh::Connect( const DevIf& _di )
{
    SetGreenLight( false );
	CommRxDev::Connect( _di );
	SetGreenLight( Connected() );
}

void 
CommRxDevTh::Connect( const IoDev& _iod )
{
    SetGreenLight( false );
	CommRxDev::Connect( _iod );
	SetGreenLight( Connected() );
}

void 
CommRxDevTh::Connect( LPCTSTR _name, DWORD _access, DWORD _create )
{
    SetGreenLight( false );
	CommRxDev::Connect( _name, _access, _create );
	SetGreenLight( Connected() );
}

void 
CommRxDevTh::Connect( LPCTSTR _name, DWORD _access, uint _br, uint _db, uint _p, uint _sb )
{
    SetGreenLight( false );
	CommRxDev::Connect( _name, _access, _br, _db, _p, _sb );
	SetGreenLight( Connected() );
}

void 
CommRxDevTh::Connect( LPCTSTR _name, DWORD _access,  char const * _str )
{
    SetGreenLight( false );
	CommRxDev::Connect( _name, _access, _str );
	SetGreenLight( Connected() );
}

void 
CommRxDevTh::Connect( LPCTSTR _name, DWORD _access,  PortSettings const & _ps )
{
    SetGreenLight( false );
	CommRxDev::Connect( _name, _access, _ps );
	SetGreenLight( Connected() );
}

void 
CommRxDevTh::DisConnect()
{
	SetGreenLight( false );
	CommRxDev::DisConnect();
}

const 
char& 
CommRxDevTh::Remove() 
{ 
	return CommRxDev::RemoveX(); 
}

uint   		
CommRxDevTh::Remove( char* _dest, uint _qty ) 
{ 
	return CommRxDev::RemoveX( _dest, _qty ); 
}

void		
CommRxDevTh::SetGreenLight( bool _gl )
{
	if ( _gl )
		portTrafficLight.Set();		// allow read thread to run
	else
		portTrafficLight.Reset();	// pause read thread
}

WDWord	
CommRxDevTh::ReadDataTh( void* )
{
	portTrafficLight.Wait();			// sleep until ok to access port
	while ( running )
	{
        ReadData();
		portTrafficLight.Wait();			// sleep until ok to access port
	}
	return 0;
}


//////////////////////////////////////////////////////////////////////////////
//
// class	CommTxDevTh methods
//

CommTxDevTh::CommTxDevTh( uint _qsze )
	: CommTxDev(_qsze)
{
	Init( Connected() );
}

CommTxDevTh::CommTxDevTh(  const DevIf& _di, uint _qsze )
	: CommTxDev(_di,_qsze)
{
	Init( Connected() );
}

CommTxDevTh::CommTxDevTh( LPCTSTR _name, DWORD _access, DWORD, uint _qsze )
	: CommTxDev(_name,_access, OPEN_EXISTING,_qsze)
{
	Init( Connected() );
}

CommTxDevTh::CommTxDevTh( const CommTxDevTh& _dev )
	: CommTxDev(_dev)
{
	Init( Connected() );
}

CommTxDevTh::CommTxDevTh( LPCTSTR _name, DWORD _access,  uint _br, uint _db, uint _p, uint _sb, uint _qsze )
	: CommTxDev(_name,_access,_br,_db,_p,_sb,_qsze)
{
	Init( Connected() );
}

CommTxDevTh::CommTxDevTh( LPCTSTR _name, DWORD _access, char* _str, uint _qsze )
	: CommTxDev(_name,_access,_str,_qsze)
{
	Init( Connected() );
}

CommTxDevTh::CommTxDevTh( LPCTSTR _name, DWORD _access, PortSettings const & _ps, uint _qsze )
	: CommTxDev(_name,_access,_ps,_qsze)
{
	Init( Connected() );
}

void
CommTxDevTh::Init( bool _connected ) 
{
	// create the semaphores 
	//WString	ws = Name();
    grabQue.Create( ""/*ws + "TxgrabQue"*/ );				
    portTrafficLight.Create( ""/*ws + "TxTrafficLight"*/ );	
    dataToXmit.Create( ""/*ws + "DataToXmit"*/ );
    	
    dataToXmit.Reset();	// no data to Xmit yet
	running = true;		// start off running
	
	// set traffic light according to connected status
	SetGreenLight( Connected() );
	
	// start the port access thread
	writeThread.Create( this, (WThreadRoutine)WriteDataTh, 0 );
	writeThread.SetPriority( WThreadPriorityAboveNormal );
}

CommTxDevTh::~CommTxDevTh() 
{
	running = false;			// tell thread to shutdown
    portTrafficLight.Set();     // stop thread waiting on semaphore
    dataToXmit.Set();			// stop thread waiting on semaphore
	writeThread.Wait();			// wait for thread to shutdown
}

void 	
CommTxDevTh::Connect( const DevIf& _di )
{
    SetGreenLight( false );
	CommTxDev::Connect( _di );
	SetGreenLight( Connected() );
}

void 
CommTxDevTh::Connect( const IoDev& _iod )
{
    SetGreenLight( false );
	CommTxDev::Connect( _iod );
	SetGreenLight( Connected() );
}

void 
CommTxDevTh::Connect( LPCTSTR _name, DWORD _access, DWORD _create )
{
    SetGreenLight( false );
	CommTxDev::Connect( _name, _access, _create );
	SetGreenLight( Connected() );
}

void 
CommTxDevTh::Connect( LPCTSTR _name, DWORD _access, uint _br, uint _db, uint _p, uint _sb )
{
    SetGreenLight( false );
	CommTxDev::Connect( _name, _access, _br, _db, _p, _sb );
	SetGreenLight( Connected() );
}

void 
CommTxDevTh::Connect( LPCTSTR _name, DWORD _access,  char const * _str )
{
    SetGreenLight( false );
	CommTxDev::Connect( _name, _access, _str );
	SetGreenLight( Connected() );
}

void 
CommTxDevTh::Connect( LPCTSTR _name, DWORD _access,  PortSettings const & _ps )
{
    SetGreenLight( false );
	CommTxDev::Connect( _name, _access, _ps );
	SetGreenLight( Connected() );
}

void 
CommTxDevTh::DisConnect()
{
	SetGreenLight( false );			// stop port access thread
	CommTxDev::DisConnect();
}

char const & 
CommTxDevTh::Add( char const & _item ) 
{ 
	dataToXmit.Set();	// have data to Xmit
	return  CommTxDev::AddX( _item );;
}

uint   		
CommTxDevTh::Add( char const * _src, uint _qty ) 
{ 
	uint	num = CommTxDev::AddX( _src, _qty );
	if ( 0 < num )
		dataToXmit.Set();	// have data to Xmit
	return  num;
}

void		
CommTxDevTh::SetGreenLight( bool _gl )
{
	if ( _gl )
		portTrafficLight.Set();		// allow read thread to run
	else
		portTrafficLight.Reset();	// pause read thread
}

WDWord	
CommTxDevTh::WriteDataTh( void* )
{
	portTrafficLight.Wait();			// sleep until ok to access port
	while ( running )
	{
        WriteData();
        if ( 0 == Count() )
        	// no more data to Xmit, so reset the 'wait on data' semaphore
			dataToXmit.Reset();		
		dataToXmit.Wait();					// sleep until have data to Xmit
		portTrafficLight.Wait();			// sleep until ok to access port
	}
	return 0;
}



