//	IoDev.hpp

#ifndef	__IODEV_HPP__
#define	__IODEV_HPP__

#include "DevIf.hpp"		// for DevIf stuff
#include "BCirQov.hpp"		// for queue stuff

//////////////////////////////////////////////////////////////////////////////
//
// class	IoDev
//
class	IoDev : public WObject
{
public:
	IoDev( uint _bsze );
	virtual ~IoDev();
	
	virtual void Connect( LPCTSTR _name, DWORD _access, DWORD _create )= 0;
	virtual void Connect( const IoDev& _iod )= 0;

	virtual DevIf& DevInterface() const = 0;

protected:	
	char*	Buf() const ;
	void	SetBuf( uint _bufSize );
	uint	BufSize() const ;
private:	
	uint				bufSize;
	char*				buf;
};

//////////////////////////////////////////////////////////////////////////////
//
// class	FileRxDev
//
class	FileRxDev : public  IoDev
{
public:
	FileRxDev( uint _qsze );
	FileRxDev(  const DevIf& _di , uint _qsze );
	FileRxDev( LPCTSTR _name, DWORD _access, DWORD _create, uint _qsze );
	FileRxDev(  const FileRxDev& _dev );
	~FileRxDev();
	
	void 			Connect( const DevIf& _di );
	virtual void 	Connect( LPCTSTR _name, DWORD _access, DWORD _create );
	virtual void 	Connect( const IoDev& _iod );

	void DisConnect() { rxDevIf.DisConnect(); }
	void ReConnect() { rxDevIf.ReConnect(); }
	
    virtual const char& Remove() ;
    virtual uint   		Remove( char* _dest, uint _qty ) ;
	virtual DevIf& 		DevInterface() const ;

	LPCTSTR		Name() const { return rxDevIf.Name(); }
	DWORD		AccessMode() const { return	rxDevIf.AccessMode(); }
	DWORD		CreateMode() const { return	rxDevIf.CreateMode(); }
	bool		Connected() const { return rxDevIf.Connected(); }

protected:
    const char& RemoveX() ;
    uint   		RemoveX( char* _dest, uint _qty ) ;
	
private:
	void	ReadData();

	RxFileDevIf			rxDevIf;	// rx device interface
	BQueueOvr<char>		rxQue;		// rx data queue
};
	
//////////////////////////////////////////////////////////////////////////////
//
// class	FileTxDev
//
class	FileTxDev : public  IoDev
{
public:
	FileTxDev( uint _qsze );
	FileTxDev(  const DevIf& _di , uint _qsze );
	FileTxDev( LPCTSTR _name, DWORD _access, DWORD _create, uint _qsze );
	FileTxDev(  const FileTxDev& _dev );
	~FileTxDev();
	
	void 	Connect( const DevIf& _di );
	virtual void 	Connect( LPCTSTR _name, DWORD _access, DWORD _create );
	virtual void 	Connect( const IoDev& _iod );
	
	void DisConnect() { txDevIf.DisConnect(); }
	void ReConnect() { txDevIf.ReConnect(); }
	
    virtual const char& Add( const char& _item ) ;
    virtual uint	    Add( const char* _src, uint _qty );

	virtual DevIf& DevInterface() const;
	
	LPCTSTR		Name() const { return txDevIf.Name(); }
	DWORD		AccessMode() const { return	txDevIf.AccessMode(); }
	DWORD		CreateMode() const { return	txDevIf.CreateMode(); }
	bool		Connected() const { return txDevIf.Connected(); }

protected:
    const char& AddX( const char& _item );
    uint	    AddX( const char* _src, uint _qty );

private:
	void	WriteData();

	TxFileDevIf		    txDevIf;	// tx device interface
	BQueueOvr<char>		txQue;		// tx data queue
};
	

//////////////////////////////////////////////////////////////////////////////
//
// class	FileRxTxDev
//
class	FileRxTxDev : public FileRxDev, public FileTxDev
{
public:
	FileRxTxDev( uint _rqsze, uint _tqsze )
		: FileRxDev(_rqsze), FileTxDev(_tqsze)
	{
	}
	
	FileRxTxDev( FileRxTxDev const & _dev  )
		: FileRxDev(_dev), FileTxDev(_dev)
	{
	}

	FileRxTxDev( LPCTSTR _name, DWORD _access, DWORD _create,  uint _rqsze, uint _tqsze  )
		: FileRxDev(_name,_access,_create,_rqsze), FileTxDev(DevInterface(),_tqsze)
	{
	}

	~FileRxTxDev() {}
	
	virtual void Connect( LPCTSTR _name, DWORD _create )
	{
		FileRxDev::Connect( _name, GENERIC_READ | GENERIC_WRITE, _create );
		FileTxDev::Connect( (FileRxDev)*this );
	}

	virtual void Connect( const IoDev& _iod )
	{
		FileRxDev::Connect( _iod );
		FileTxDev::Connect( _iod );
	}

    virtual DevIf& DevInterface() const { return FileRxDev::DevInterface(); }

	LPCTSTR		Name() const { return FileRxDev::Name(); }
	DWORD		AccessMode() const { return	FileRxDev::AccessMode(); }
	DWORD		CreateMode() const { return	FileRxDev::CreateMode(); }
	bool		Connected() const { return FileRxDev::Connected(); }

private:
	virtual void Connect( LPCTSTR _name, DWORD _access, DWORD _create )
	{
		FileRxDev::Connect( _name, _access, _create );
		FileTxDev::Connect( (FileRxDev)*this );
	}

};



//////////////////////////////////////////////////////////////////////////////
//
// class	CommRxDev
//
class	CommRxDev : public IoDev
{
public:
	CommRxDev( uint _qsze );
	CommRxDev(  const DevIf& _di , uint _qsze );
	CommRxDev( LPCTSTR _name, DWORD _access, DWORD , uint _qsze );
	CommRxDev( LPCTSTR _name, DWORD _access, uint _br, uint _db, uint _p, uint _sb, uint _qsze );
	CommRxDev( LPCTSTR _name, DWORD _access, char* _str, uint _qsze );
	CommRxDev( LPCTSTR _name, DWORD _access, PortSettings const & _ps, uint _qsze );
	CommRxDev(  const CommRxDev& _dev );
	~CommRxDev();
	
	void Connect( const DevIf& _di );
	virtual void Connect( LPCTSTR _name, DWORD _access, DWORD _create=OPEN_EXISTING );
	virtual void Connect( const IoDev& _iod );
	virtual void Connect( LPCTSTR _name, DWORD _access, uint _br, uint _db, uint _p, uint _sb );
	virtual void Connect( LPCTSTR _name, DWORD _access, char const * _str );
	virtual void Connect( LPCTSTR _name, DWORD _access, PortSettings const & _ps );

	void DisConnect() { rxDevIf.DisConnect(); }
	void ReConnect() { rxDevIf.ReConnect(); }
	
    virtual char const & 	Remove() ;
    virtual uint   			Remove( char* _dest, uint _qty );

	virtual DevIf& 	DevInterface() const ;

	LPCTSTR		Name() const { return rxDevIf.Name(); }
	DWORD		AccessMode() const { return	rxDevIf.AccessMode(); }
	DWORD		CreateMode() const { return	rxDevIf.CreateMode(); }
	bool		Connected() const { return rxDevIf.Connected(); }

protected:
    char const & 	RemoveX();
    uint   			RemoveX( char* _dest, uint _qty );
	void			ReadData();
	
private:

	RxCommDevIf			rxDevIf;	// rx device interface
	BQueueOvr<char>		rxQue;		// rx data queue
};

	
//////////////////////////////////////////////////////////////////////////////
//
// class	CommTxDev
//
class	CommTxDev : public IoDev
{
public:
	CommTxDev( uint _qsze );
	CommTxDev(  const DevIf& _di , uint _qsze );
	CommTxDev( LPCTSTR _name, DWORD _access, DWORD , uint _qsze );
	CommTxDev( LPCTSTR _name, DWORD _access,  uint _br, uint _db, uint _p, uint _sb, uint _qsze );
	CommTxDev( LPCTSTR _name, DWORD _access,  char* _str, uint _qsze );
	CommTxDev( LPCTSTR _name, DWORD _access,  PortSettings const & _ps, uint _qsze );
	CommTxDev(  const CommTxDev& _dev );

	~CommTxDev();
	
	void Connect( const DevIf& _di );
	virtual void Connect( LPCTSTR _name, DWORD _access, DWORD _create=OPEN_EXISTING );
	virtual void Connect( const IoDev& _iod );
	virtual void Connect( LPCTSTR _name, DWORD _access, uint _br, uint _db, uint _p, uint _sb );
	virtual void Connect( LPCTSTR _name, DWORD _access, char const * _str );
	virtual void Connect( LPCTSTR _name, DWORD _access, PortSettings const & _ps );
	
	void DisConnect() { txDevIf.DisConnect(); }
	void ReConnect() { txDevIf.ReConnect(); }
	
    virtual const char& Add( const char& _item );
    virtual uint	    Add( const char* _src, uint _qty );

	virtual DevIf& DevInterface() const;
	
	LPCTSTR		Name() const { return txDevIf.Name(); }
	DWORD		AccessMode() const { return	txDevIf.AccessMode(); }
	DWORD		CreateMode() const { return	txDevIf.CreateMode(); }
	bool		Connected() const { return txDevIf.Connected(); }

	uint		Count() const { return txQue.Count(); }
	
protected:
    const char& AddX( const char& _item );
    uint	    AddX( const char* _src, uint _qty );
	void		WriteData();

private:

	TxCommDevIf		    txDevIf;	// tx device interface
	BQueueOvr<char>		txQue;		// tx data queue
};
	

//////////////////////////////////////////////////////////////////////////////
//
// class	CommRxTxDev
//
class	CommRxTxDev : public CommRxDev, public CommTxDev
{
public:
	CommRxTxDev( uint _rqsze, uint _tqsze )
		: CommRxDev(_rqsze), CommTxDev(_tqsze)
	{
	}
	
	CommRxTxDev( CommRxTxDev const & _dev  )
		: CommRxDev(_dev), CommTxDev(_dev)
	{
	}

	CommRxTxDev( LPCTSTR _name, uint _rqsze, uint _tqsze )
		:   CommRxDev(_name,GENERIC_READ | GENERIC_WRITE,OPEN_EXISTING,_rqsze), 
            CommTxDev(DevInterface(),_tqsze)
	{
	}

	CommRxTxDev( LPCTSTR _name, uint _br, uint _db, uint _p, uint _sb,  
				uint _rqsze, uint _tqsze )
		: 	CommRxDev(_name,GENERIC_READ | GENERIC_WRITE,_br,_db,_p,_sb,_rqsze), 
			CommTxDev(DevInterface(),_tqsze)
	{
	}

	CommRxTxDev( LPCTSTR _name, char* _str, uint _rqsze, uint _tqsze )
		: CommRxDev(_name,GENERIC_READ | GENERIC_WRITE,_str,_rqsze), CommTxDev(DevInterface(),_tqsze)
	{
	}

	CommRxTxDev( LPCTSTR _name, PortSettings const & _ps,  
				uint _rqsze, uint _tqsze )
		: 	CommRxDev(_name,GENERIC_READ | GENERIC_WRITE,_ps,_rqsze), 
			CommTxDev(DevInterface(),_tqsze)
	{
	}


	~CommRxTxDev() {}
	
	virtual void Connect( LPCTSTR _name )
	{
		CommRxDev::Connect( _name, GENERIC_READ | GENERIC_WRITE );
		CommTxDev::Connect( DevInterface() );
	}

	virtual void Connect( LPCTSTR _name, uint _br, uint _db, uint _p, uint _sb )
	{
		CommRxDev::Connect( _name, GENERIC_READ | GENERIC_WRITE, _br, _db, _p, _sb );
		CommTxDev::Connect( DevInterface() );
	}

	virtual void Connect( LPCTSTR _name, char const * _str )
	{
		CommRxDev::Connect( _name, GENERIC_READ | GENERIC_WRITE, _str );
		CommTxDev::Connect( DevInterface() );
	}

	virtual void Connect( LPCTSTR _name, PortSettings const & _ps )
	{
		CommRxDev::Connect( _name, GENERIC_READ | GENERIC_WRITE, _ps );
		CommTxDev::Connect( DevInterface() );
	}

	virtual void Connect( const IoDev& _iod )
	{
		CommRxDev::Connect( _iod );
		CommTxDev::Connect( _iod );
	}

    virtual DevIf& DevInterface() const { return CommRxDev::DevInterface(); }
    
	LPCTSTR		Name() const { return CommRxDev::Name(); }
	DWORD		AccessMode() const { return	CommRxDev::AccessMode(); }
	DWORD		CreateMode() const { return	CommRxDev::CreateMode(); }
	bool		Connected() const { return CommRxDev::Connected(); }

private:
	virtual void Connect( LPCTSTR, DWORD, DWORD ) {}
	virtual void Connect( LPCTSTR, DWORD, uint, uint, uint, uint ) {}
	virtual void Connect( LPCTSTR, DWORD, char const * ) {}
	virtual void Connect( LPCTSTR, DWORD, PortSettings const & ) {}
    
};


//////////////////////////////////////////////////////////////////////////////
//
// class	CommRxDevTh
//
class	CommRxDevTh : public CommRxDev
{
public:
	CommRxDevTh( uint _qsze );
	CommRxDevTh(  const DevIf& _di , uint _qsze );
	CommRxDevTh( LPCTSTR _name, DWORD _access, DWORD , uint _qsze );
	CommRxDevTh( LPCTSTR _name, DWORD _access, uint _br, uint _db, uint _p, uint _sb, uint _qsze );
	CommRxDevTh( LPCTSTR _name, DWORD _access, char* _str, uint _qsze );
	CommRxDevTh( LPCTSTR _name, DWORD _access, PortSettings const & _ps, uint _qsze );
	CommRxDevTh(  const CommRxDevTh& _dev );

	~CommRxDevTh();
	
	void Connect( const DevIf& _di );
	virtual void Connect( LPCTSTR _name, DWORD _access, DWORD _create=OPEN_EXISTING );
	virtual void Connect( const IoDev& _iod );
	virtual void Connect( LPCTSTR _name, DWORD _access, uint _br, uint _db, uint _p, uint _sb );
	virtual void Connect( LPCTSTR _name, DWORD _access, char const * _str );
	virtual void Connect( LPCTSTR _name, DWORD _access, PortSettings const & _ps );

	virtual void DisConnect();
	
    virtual char const & 	Remove() ;
    virtual uint   			Remove( char* _dest, uint _qty );

	void		SetGreenLight( bool _gl );

private:
	WDWord	ReadDataTh( void* );
	void	Init( bool _connected );

	WThread					readThread;		// read from port thread
	bool					running;		// true if read thread is running
    WMutexSemaphore         grabQue;		// coordinates multi thread access to the queue
	WEventSemaphore 		portTrafficLight;	// set if ok to access port, else reset
};

	
//////////////////////////////////////////////////////////////////////////////
//
// class	CommTxDevTh
//
class	CommTxDevTh : public CommTxDev
{
public:
	CommTxDevTh( uint _qsze );
	CommTxDevTh(  const DevIf& _di , uint _qsze );
	CommTxDevTh( LPCTSTR _name, DWORD _access, DWORD , uint _qsze );
	CommTxDevTh( LPCTSTR _name, DWORD _access, uint _br, uint _db, uint _p, uint _sb, uint _qsze );
	CommTxDevTh( LPCTSTR _name, DWORD _access, char* _str, uint _qsze );
	CommTxDevTh( LPCTSTR _name, DWORD _access, PortSettings const & _ps, uint _qsze );
	CommTxDevTh(  const CommTxDevTh& _dev );

	~CommTxDevTh();
	
	void Connect( const DevIf& _di );
	virtual void Connect( LPCTSTR _name, DWORD _access, DWORD _create=OPEN_EXISTING );
	virtual void Connect( const IoDev& _iod );
	virtual void Connect( LPCTSTR _name, DWORD _access, uint _br, uint _db, uint _p, uint _sb );
	virtual void Connect( LPCTSTR _name, DWORD _access, char const * _str );
	virtual void Connect( LPCTSTR _name, DWORD _access, PortSettings const & _ps );

	virtual void DisConnect();
	
    virtual const char& Add( const char& _item );
    virtual uint	    Add( const char* _src, uint _qty );

	void		SetGreenLight( bool _gl );

private:
	WDWord	WriteDataTh( void* );
	void	Init( bool _connected );

	WThread					writeThread;		// write to port thread
	bool					running;			// true if read thread is running
    WMutexSemaphore         grabQue;			// coordinates multi thread access to the queue
	WEventSemaphore 		portTrafficLight;	// set if ok to access port, else reset
	WEventSemaphore 		dataToXmit;			// set if have data to Xmit, else reset
};

	
//////////////////////////////////////////////////////////////////////////////
//
// class	CommRxTxDevTh
//
class	CommRxTxDevTh : public CommRxDevTh, public CommTxDevTh
{
public:
	CommRxTxDevTh( uint _rqsze, uint _tqsze )
		: CommRxDevTh(_rqsze), CommTxDevTh(_tqsze)
	{
	}
	
	CommRxTxDevTh( CommRxTxDevTh const & _dev  )
		: CommRxDevTh(_dev), CommTxDevTh(_dev)
	{
	}

	CommRxTxDevTh( LPCTSTR _name, uint _rqsze, uint _tqsze )
		:   CommRxDevTh(_name,GENERIC_READ | GENERIC_WRITE,OPEN_EXISTING,_rqsze), 
            CommTxDevTh(DevInterface(),_tqsze)
	{
	}

	CommRxTxDevTh( LPCTSTR _name, uint _br, uint _db, uint _p, uint _sb,  
				uint _rqsze, uint _tqsze )
		: 	CommRxDevTh(_name,GENERIC_READ | GENERIC_WRITE,_br,_db,_p,_sb,_rqsze), 
			CommTxDevTh(DevInterface(),_tqsze)
	{
	}

	CommRxTxDevTh( LPCTSTR _name, char* _str, uint _rqsze, uint _tqsze )
		: CommRxDevTh(_name,GENERIC_READ | GENERIC_WRITE,_str,_rqsze), CommTxDevTh(DevInterface(),_tqsze)
	{
	}

	CommRxTxDevTh( LPCTSTR _name, PortSettings const & _ps,  
				uint _rqsze, uint _tqsze )
		: 	CommRxDevTh(_name,GENERIC_READ | GENERIC_WRITE,_ps,_rqsze), 
			CommTxDevTh(DevInterface(),_tqsze)
	{
	}


	virtual ~CommRxTxDevTh() {}
	
	virtual void Connect( LPCTSTR _name )
	{
		//DisConnect();
		CommRxDevTh::Connect( _name, GENERIC_READ | GENERIC_WRITE );
		CommTxDevTh::Connect( DevInterface() );
	}

	virtual void Connect( LPCTSTR _name, uint _br, uint _db, uint _p, uint _sb )
	{
		//DisConnect();
		CommRxDevTh::Connect( _name, GENERIC_READ | GENERIC_WRITE, _br, _db, _p, _sb );
		CommTxDevTh::Connect( DevInterface() );
	}

	virtual void Connect( LPCTSTR _name, char const * _str )
	{
		DisConnect();
        
        WCurrentThread  th;
        th.Wait( 50 );		// wait for op sys to fully release handle
        
		CommRxDevTh::Connect( _name, GENERIC_READ | GENERIC_WRITE, _str );
		CommTxDevTh::Connect( DevInterface() );
	}

	virtual void Connect( LPCTSTR _name, PortSettings const & _ps )
	{
		DisConnect();
        
        WCurrentThread  th;
        th.Wait( 50 );		// wait for op sys to fully release handle
        
		CommRxDevTh::Connect( _name, GENERIC_READ | GENERIC_WRITE, _ps );
		CommTxDevTh::Connect( DevInterface() );
	}

	virtual void Connect( const IoDev& _iod )
	{
		CommRxDevTh::Connect( _iod );
		CommTxDevTh::Connect( _iod );
	}

	void	DisConnect()
	{
		CommRxDevTh::DisConnect();
		CommTxDevTh::DisConnect();
	}
	
	void	SetGreenLight( bool _gl )
	{
		CommRxDevTh::SetGreenLight( _gl );	// set for Rx device
		CommTxDevTh::SetGreenLight( _gl );	// and set for Tx device
	}

    virtual DevIf& DevInterface() const { return CommRxDevTh::DevInterface(); }
    
	LPCTSTR		Name() const { return CommRxDevTh::Name(); }
	DWORD		AccessMode() const { return	CommRxDevTh::AccessMode(); }
	DWORD		CreateMode() const { return	CommRxDevTh::CreateMode(); }
	bool		Connected() const { return CommRxDevTh::Connected(); }

private:
	virtual void Connect( LPCTSTR, DWORD, DWORD ) {}
	virtual void Connect( LPCTSTR, DWORD, uint, uint, uint, uint ) {}
	virtual void Connect( LPCTSTR, DWORD, char const * ) {}
	virtual void Connect( LPCTSTR, DWORD, PortSettings const & ) {}
    
};



#include "IoDev.cpp"		// until create library

	
#endif	// __IODEV_HPP__
