//	PortSet.cpp

#include "PortSet.hpp"
#include <strstream.h>

//////////////////////////////////////////////////////////////////////////////
//
// class	PortSettings methods
//

PortSettings::PortSettings( uint _br, uint _db, uint _p, uint _sb )
{
	Set( _br, _db, _p, _sb );
}

PortSettings::PortSettings( char* _str )
{
	Set( _str );
}

PortSettings& 
PortSettings::operator = ( const PortSettings& _ps )
{
 	Set(  _ps.baudRate, _ps.dataBits, _ps.parity, _ps.stopBits );
	return *this;
}

void
PortSettings::Set( uint _br )
{
	baudRate  	= _br;
}

void
PortSettings::Set( uint _br, uint _db )
{
	Set( _br );
	dataBits  	= _db;
}

void
PortSettings::Set( uint _br, uint _db, uint _p )
{
	Set( _br, _db );
	parity		= _p;
}

void
PortSettings::Set( uint _br, uint _db, uint _p, uint _sb )
{
	Set( _br, _db, _p );
	stopBits	= _sb;	
}

void
PortSettings::Set( char const * _str )
{
	long	br=-1, db=-1, p=-1, sb=-1;
	
    istrstream   ds( (char*)_str, strlen(_str) );

	int		pos = ds.tellg();
	skipNonDigitsAnd1stZero( ds );		ds >> br;      	pos = ds.tellg();
	skipNonDigitsAnd1stZero( ds );		ds >> db;     	pos = ds.tellg();
	skipNonDigitsAnd1stZero( ds );		ds >> p;     	pos = ds.tellg();
	skipNonDigitsAnd1stZero( ds );		ds >> sb;		pos = ds.tellg();
	if ( -1 != sb )	
		Set( br, db, p, sb );
	else if ( -1 != p )		
		Set( br, db, p );
	else if ( -1 != db )		
		Set( br, db );
	else if ( -1 != br )		
		Set( br );
}

const char*       
PortSettings::String() const
{
    strstream   ds( stringBuf, StringLen );
    
    ds  << baudRate << ',' << dataBits << ',' << parity << ',' << stopBits;
    ds << ends; 
    return  stringBuf;
}


const char*
PortSettings::IniText()
{
	return "Baud rate,[Data bits=8,Parity=0 (0=N,1=O,2=E),Stop bits=1]";
}

void
PortSettings::skipNonDigitsAnd1stZero( istream& _is )
//
// Skips to 1st numeric digit in the stream.
// If  it is a '0' & the next char is also numeric, removes the leading zero
// to prevent conversion to octal.
//
{
    char    c1, c2;
    
    while ( _is )
    {
        c1 = _is.get();
        if ( isdigit( c1 ) )
        {
            // are at first numeric char
            if ( '0' == c1 )
            {
                // 1st numeric char is '0'
                _is.get( c2 );
                if ( isdigit( c2 ) )
                {
                    // 2nd char is numeric, so strip the leading '0'
                    _is.putback( c2 );
                }
                else
                {
                    // 2nd char is Not numeric, so do NOT strip the leading '0'
                    _is.putback( c2 );
                    _is.putback( c1 );
                }
            }
            else
                // numeric digit is not '0' so just put it back
                _is.putback( c1 );
            break;
        }
    }
}

