//	BQueue.hpp

#ifndef __BQUEUE__HPP__
#define __BQUEUE__HPP__

#include "Queue.hpp"

#ifdef USING
namespace ktUtil
{
#endif

//===========================================================================//
//  class BQueue
//
//  Provides the std interface for a bounded queue.
//	Derived from class Queue
//	This is an abstract class, all of the methods are pure virtual
//
//  Public Methods:
//	---------------
//                  
//  destructor      
//		Virtual destructor, for polymorphic use.
//
// const T& Add( const T& _item )
//		Adds a single element to the queue, returns reference to the element added
//
// void	 	Add( T* _src, uint _qty )
//		Adds _qty elements to queue, from array _src.
//		Returns no. of elements added. May throw QueueException.
//
// const T&   	Remove()
//		Removes & returns a single element from the queue.
//		If queue is empty throws a QueueException 
//
// uint 	Remove( T* _dest, uint _qty )
//		Removes up to _qty elements from queue & places them into _dest.
//		Returns the actual no. of elements removed. May throw QueueException.
//
// T   		Read( uint& _pos )
// 		Returns copy of elmnt at posn _pos in queue
// 		Updates _pos to indicate next element (if any) in queue. NOTE _pos must be a var.
// 		Does Not wrap around past end of queue.
// 		If _posn indicates element outside queue, throws a QueueException
//
// T  		ReadX( const uint& _pos )
// 		Returns copy of elmnt at posn _pos in queue
// 		Does Not wrap around past end of queue.
// 		If _posn indicates element outside queue, throws a QueueException
//		Wrapper for Read(). Can be called with constant for _pos
//
// uint		Read( T* _dest, uint _qty, uint& _pos )
// 		Copies up to _qty elements from elmnt at posn _pos in queue into _dest
// 		Returns actual no. of elements read.
// 		Updates _pos to indicate 1st element (if any) not read from queue, allowing
// 		reading queue in small steps. NOTE _pos must be a var.
// 		Does NOT wrap around past end of queue
//
// uint   	ReadX( T* _dest, uint _qty, const uint& _pos )
// 		Copies up to _qty elements from elmnt at posn _pos in queue into _dest
// 		Returns actual no. of elements read.
// 		reading queue in small steps.
// 		Does NOT wrap around past end of queue
//		Wrapper for Read(). Can be called with constant for _pos
//
// void		Empty()		// Empties the queue.
// uint		Count()		// No. of elements currently in thew queue
// uint		Capacity()	// No. of elements that the queue can hold
// uint		FreeSpace()	// No. of elmnts that can be added w/o over writing data.
// uint		Begin()		// Offset of first element in queue
// uint		End()		// Offset of last element in queue
// uint		Count( uint _pos )	
// 				no. of elements from posn _pos to end of queue
//
//===========================================================================//
//
template<class T>
class   BQueue : public Queue<T>
{
public:
    
    virtual ~BQueue() {}
    
    virtual const T&    Add( const T& _item ) =0;
    virtual uint	    Add( T const * _src, uint _qty ) =0;

    virtual const T&   	Remove() =0;
    virtual uint   		Remove( T* _dest, uint _qty ) =0;
	virtual void		Empty() = 0;

    virtual const T&    Read( uint& _pos ) =0;
    virtual const T&    ReadX( const uint& _pos ) =0;
    virtual uint   		Read( T* _dest, uint _qty, uint& _pos ) =0;
    virtual uint   		ReadX( T* _dest, uint _qty, const uint& _pos ) =0;

	virtual uint		Count()	const =0;
	virtual uint		Capacity()	const =0;
	virtual uint		FreeSpace() const =0;
	virtual uint		Count( uint _pos ) const =0;
	virtual uint		Begin() const =0;
	virtual uint		End() const =0;
};


#ifdef USING
}		// namespace ktUtil
#endif

#endif  // __QUEUE__HPP__
