//  Bcirq_no.hpp
//
//  Declaration for BCircQ_no class

#ifndef __BCIRCQ_NO__HPP__
#define __BCIRCQ_NO__HPP__


#include "Bcirq_o.hpp"

#ifdef USING
namespace ktUtil
{
#endif

//===========================================================================//
//  class BCircQ_no
//
//  Implements a bounded circular queue, NOT allowing overwrites
//
//	Derived from class BBCircQ_no
//	The queue does not allow existing data to be overwritten by new data.
//	Attempting to remove data from an empty queue will throw a QueueException
//	Attempting to over fill the queue may throw a QueueException
//
//  Public Methods:
//	---------------
//  constructor     
//		Accepts 1 arg which specifies the queue size.
//      Dynamically allocates memory for the queue buffer.
// 		If unable to allocate memory for queue store throws a QueueException
//  destructor      Deletes the memory buffer used by the queue
//
// const T& Add( const T& _item )
//		Adds a single element to the queue, returns the element added
//		If queue is full throws a QueueException
//
// void	 	Add( T* _src, uint _qty )
//		Adds _qty elements to queue, from array _src.
//		Only adds enough elements to just fill the queue
//		Returns no. of elements added, does NOT throw QueueException.
//
//	Inherited Public Methods
//	------------------------
// const T&   	Remove()
//		Removes & returns a single element from the queue.
//		If queue is empty throws a QueueException 
//
// uint 	Remove( T* _dest, uint _qty )
//		Removes up to _qty elements from queue & places them into _dest.
//		Returns the actual no. of elements removed.
//		If queue is empty throws a QueueException
//
// const T&	Read( uint& _pos )
// 		Returns copy of elmnt at posn _pos in queue
// 		Updates _pos to indicate next element (if any) in queue. NOTE _pos must be a var.
// 		Does Not wrap around past end of queue.
// 		If _posn indicates element outside queue, throws a QueueException
//
// const T&	ReadX( const uint& _pos )
//		Wrapper for Read(). Can be called with constant for _pos
// 		Returns copy of elmnt at posn _pos in queue
// 		Does Not wrap around past end of queue.
// 		If _posn indicates element outside queue, throws a QueueException
//
// uint		Read( T* _dest, uint _qty, uint& _pos )
// 		Copies up to _qty elements from elmnt at posn _pos in queue into _dest
// 		Returns actual no. of elements read.
// 		Updates _pos to indicate 1st element (if any) not read from queue, allowing
// 		reading queue in small steps. NOTE _pos must be a var.
// 		Does NOT wrap around past end of queue
//
// uint   	ReadX( T* _dest, uint _qty, const uint& _pos )
//		Wrapper for Read(). Can be called with constant for _pos
// 		Copies up to _qty elements from elmnt at posn _pos in queue into _dest
// 		Returns actual no. of elements read.
// 		reading queue in small steps.
// 		Does NOT wrap around past end of queue
//
// void		ClearUnused()
// 		resets all unused elements to 0.
// 		Provided for derived classes which store ptrs in queue.
// 		Setting ptr to 0 before deletion will prevent heap corruption
//
// uint		Count()		//	No. of elements currently in thew queue
// uint		Capacity()	//	No. of elements that the queue can hold
// uint		Space()		//	No. of elements that can be safely added to queue.
//
//===========================================================================//
template<class T>
class   BCircQ_no : public BCircQ_o<T>
{
public:
    BCircQ_no( uint _size )	: BCircQ_o(_size) {}

    virtual ~BCircQ_no() {}
    
    const T&    Add( const T& _item )
	//
	// Adds 1 element to the queue, if there is room
	//
	{
		if ( Capacity() <= Count() )
			throw QueueException(QueueException::FullQ);		// full queue
			
		return BCircQ_o::Add( _item );
	}
	
    uint	    Add( T const * _src, uint _qty )
	//
	// Adds up to _qty elements from _src into the queue
	// will never overfill the queue, ie overwrite existing queue data
	// Returns actual no. of elements added.
	//
	{
		uint	room  = FreeSpace();
		if ( _qty > room )
		{
			// trying to add more than queue can hold.
			// So just add enough to just fill the queue
			_qty = room;
		}
		if ( 0 < _qty )
			// ok add _qty elmnts to the queue
			return BCircQ_o::Add( _src, _qty );
		return 0;	// queue is full 
	}

};

#ifdef USING
}		// namespace ktUtil
#endif

//#include "Bcirq_no.cpp"

#endif  // __BCIRCQ_NO__HPP__
