//  CircCntr.hpp
//
//  Definition of CircCntr  & CircPtr<T> classes

#ifndef __CIRCCNTR__HPP__
#define __CIRCCNTR__HPP__

#ifndef  uint
typedef unsigned int  uint;
#endif

#ifdef USING
namespace ktUtil
{
#endif

///////////////////////////////////////////////////////////////////////////
//	CircCntr
//
class	CircCntr
{
public:
	explicit CircCntr( uint _maxCnt ) : MAX_CNT(_maxCnt), value(0) {}
	CircCntr( uint _maxCnt, uint _val ) : MAX_CNT(_maxCnt), value(_val%MAX_CNT) {}
	CircCntr( const CircCntr& _cntr, uint _val ) 
		: MAX_CNT(_cntr.MAX_CNT), value(_cntr.value) { Inc(_val); }
	CircCntr( const CircCntr& _cntr ) : MAX_CNT(_cntr.MAX_CNT), value(_cntr.value) {}
	
	const CircCntr&	operator = ( const CircCntr& _cntr ) 
		{ value = _cntr.value % MAX_CNT; return *this; }
	const CircCntr&	operator = ( uint _val ) 
		{ value = _val % MAX_CNT; return *this; }
	CircCntr&	operator += ( int _val ) { Inc( _val ); return *this; }
	CircCntr&	operator -= ( int _val ) { Inc( -_val ); return *this; }
	
	CircCntr&	operator ++ () { Inc( 1 ); return *this; }
	CircCntr	operator ++ (int) { CircCntr tmp = *this; Inc(1); return tmp; }
	CircCntr&	operator -- () { Inc( -1 ); return *this; }
	CircCntr	operator -- (int) { CircCntr tmp = *this; Inc(-1); return tmp; }
	
	uint	operator + ( const CircCntr& _cntr ) 
	{ 
		CircCntr tmp = *this; tmp.Inc( _cntr.value ); return tmp.value; 
	}
	uint	operator - ( const CircCntr& _cntr ) 
	{ 
		CircCntr tmp = *this; tmp.Inc( -_cntr.value ); return tmp.value; 
	}

	operator const uint () const	{ return value; }
	operator const int () const	{ return value; }
	
	uint		MaxCnt() const { return MAX_CNT ; }
	uint		Value() const { return value ; }
private:
	void	Inc( int _val )
	{
		int	val = value;	// use signed arithmetic, not unsigned
		val += _val;
		if ( 0 > val )
		{
			// fix -ve values by doing roll around
			val = -val % MAX_CNT;
			value = MAX_CNT - val;
		}
		else
			value = val % MAX_CNT;
	}
	
	static uint	abs( int _val ) { return _val > 0 ? _val : -_val; }
			
	const uint	MAX_CNT;
	uint		value;
	
};

///////////////////////////////////////////////////////////////////////////
//	CircPtr
//
template<class T>
class	CircPtr 
{
public:
	CircPtr( const T* _ptr, uint _maxCnt) : cntr(_maxCnt), B_PTR(_ptr) {}
	CircPtr( const T* _ptr, uint _maxCnt, uint _val ) 
		: cntr(_maxCnt,_val) , B_PTR(_ptr) {}
	CircPtr( const CircPtr& _cptr, uint _val ) 
		: cntr(_cptr.cntr,_val), B_PTR(_cptr.B_PTR) {}
	CircPtr( const CircPtr& _cptr ) 
		: cntr(_cptr.cntr), B_PTR(_cptr.B_PTR) {}
	
	const CircPtr&	operator = ( const CircPtr& _cptr ) 
		{ cntr = _cptr.cntr; return *this; }
	const CircPtr&	operator = ( uint _val ) 
		{ cntr = _val; return *this; }

	CircPtr&	operator += ( int _val ) { cntr += _val; return *this; }
	CircPtr&	operator -= ( int _val ) { cntr += _val; return *this; }
	
	CircPtr&	operator ++ () { ++cntr; return *this; }
	CircPtr		operator ++ (int) { CircPtr tmp = *this; ++cntr; return tmp; }
	CircPtr&	operator -- () { --cntr; return *this; }
	CircPtr		operator -- (int) { CircPtr tmp = *this; --cntr; return tmp; }
	
	CircPtr	operator + ( int _val ) 
	{ 
		CircPtr tmp = *this; tmp += _val; return tmp; 
	}
	CircPtr	operator - ( int _val ) 
	{ 
		CircPtr tmp = *this; tmp -= _val; return tmp; 
	}

	operator const T* () const	{ return B_PTR + cntr.Value(); }
	const T operator [] ( uint _off ) const 
	{ 
		CircPtr tmp = *this; tmp += _off; return *(tmp);
	}
	
	uint		MaxOffset() const { return cntr.MaxCnt()-1; }
	uint		Offset() const { return cntr.Value(); }
private:
	
	CircCntr	cntr;
	const T*	B_PTR;
};

#ifdef USING
}				// namespace ktUtil
#endif

#endif  // __CIRCCNTR__HPP__
