//	Queue.hpp

#ifndef __QUEUE__HPP__
#define __QUEUE__HPP__

#include "QueEx.hpp"

#ifdef USING
namespace ktUtil
{
#endif

//===========================================================================//
//  class Queue
//
//  Provides the std interface for a queue.
//	This is an abstract class, all of the methods are pure virtual
//
//  Public Methods:
//	---------------
//                  
//  destructor      
//		Virtual destructor, for polymorphic use.
//
// const T& Add( const T& _item )
//		Adds a single element to the queue, returns reference to the element added
//
// void	 	Add( T* _src, uint _qty )
//		Adds _qty elements to queue, from array _src.
//		Returns no. of elements added. May throw QueueException.
//
// const T&   	Remove()
//		Removes & returns a single element from the queue.
//		If queue is empty throws a QueueException 
//
// void		Empty()		//	Empties the queue.
// uint		Count()		//	No. of elements currently in thew queue
// uint		Capacity()	//	No. of elements that the queue can hold
// uint		FreeSpace()	//	No. of elmnts that can be added w/o over writing data.
//
//===========================================================================//
//
template<class T>
class   Queue
{
public:
    
    virtual ~Queue() {}
    
    virtual const T&    Add( const T& _item ) =0;
    virtual uint	    Add( T const * _src, uint _qty ) =0;

    virtual const T&   	Remove() =0;
    virtual uint   		Remove( T* _dest, uint _qty ) =0;
	virtual void		Empty() = 0;
	
	virtual uint		Count()	const =0;
	virtual uint		Capacity()	const =0;
	virtual uint		FreeSpace() const =0;
};


#ifdef USING
}		// namespace ktUtil
#endif

#endif  // __QUEUE__HPP__
