//	QueueP.hpp
//
//  Declaration for QueuePtr class

#ifndef __QUEUEP__HPP__
#define __QUEUEP__HPP__


#ifdef USING
namespace ktUtil
{
#endif

//===========================================================================//
//  class QueuePtr
//
//  Provides the std interface for a queue that contains pointers to objects.
//	This is an abstract class, all of the methods are pure virtual
//
//  Public Methods:
//	---------------
//  destructor      
//		Virtual destructor, for polymorphic use.
//
// T* Add( const T& _item )
//		Adds a single element to the queue, returns reference to the element added
//
// void	 	Add( T** _src, uint _qty )
//		Adds _qty elements to queue, from array _src.
//		Returns no. of elements added. May throw QueueException.
//
// T* Remove()
//		Removes & returns a single element from the queue.
//		If queue is empty throws a QueueException 
//
// void	Empty()			//	Empties the queue.
// uint		Count()		//	No. of elements currently in thew queue
// uint		Capacity()	//	No. of elements that the queue can hold
// uint		FreeSpace()	//	No. of elmnts that can be added w/o over writing data.
//
//===========================================================================//
//
template<class T>
class   QueuePtr
{
public:
    
    virtual ~QueuePtr() {}
    
    virtual T const *	Add( T const * _item ) =0;
    virtual uint		Add( T const *  const * _src, uint _qty ) =0;

    virtual T const *	Remove() =0;
    virtual uint   		Remove( T const * * _dest, uint _qty ) =0;
	virtual void		Empty() = 0;

	virtual uint	Count()	const =0;
	virtual uint	Capacity()	const =0;
	virtual uint	FreeSpace() const =0;
};

#ifdef USING
} 		// namespace ktUtil		// #endif
#endif

#endif  // __QUEUEP__HPP__
