//  circq_no.hpp
//
//  Declaration of CircQ_no class

#ifndef __CIRCQ_NO__HPP__
#define __CIRCQ_NO__HPP__


#include "circq_o.hpp"

//===========================================================================//
//  class CircQ_no
//  Implements a simple circular queue, NOT allowing overwrites
//	Derived from class CirQ_o
//
//	The queue does not allow existing data to be overwritten by new data.
//	Attempting to remove data from an empty queue will throw a CircQException
//	Attempting to over fill the queue may throw a CircQException
//
//  Public Methods:
//	---------------
//  constructor     Accepts 1 arg which specifies the queue size.
//                  Dynamically allocates memory for the queue buffer.
//  destructor      Deletes the memory buffer used by the queue
//
// const T& Add( const T& _item )
//		Adds a single element to the queue, returns the element added
// 		If queue is full throws a CircQException
//
// void	 	Add( T* _src, uint _qty )
//		Adds _qty elements to queue, from array _src.
//		Only adds enough elements to just fill the queue
//		Returns no. of elements added, does NOT throw CircQException.
//
//	Inherited Public Methods
//	------------------------
// T&   	Remove()
//		Removes & returns a single element from the queue.
//		If queue is empty throws a CircQException 
//
// uint 	Remove( T* _dest, uint _qty )
//		Removes up to _qty elements from queue & places them into _dest.
//		Returns the actual no. of elements removed.
//		If queue is empty throws a CircQException
//
// uint		Count()		//	No. of elements currently in thew queue
// uint		Capacity()	//	No. of elements that the queue can hold
// uint		Space()		//	No. of elements that can be safely added to queue.
//
//===========================================================================//
template<class T>
class   CircQ_no : public CirQ_o<T>
{
public:
    CircQ_no( uint _size );
    virtual ~CircQ_no();
    
    virtual const T&    Add( const T& _item );
    virtual uint	    Add( T* _src, uint _qty );
};

#include "circq_no.cpp"

#endif  // __CIRCQ_NO__HPP__
